/* 	bin2hex.c	(c) by bill beech, 2008

DESCRIPTION
	converts binary file (*.bin) to intel hex format file (*.hex).

MODIFICATION HISTORY
	07 Jan 89 -- original
	22 May 08 -- modified for uniformity with other tools
Removed	28 Sep 09 -- modified to allow any file name
	NOv 2014 HRJ - added 2nd argument for address default 0000H
*/

#include	<stdio.h>
#include	<string.h>
#include	<stdlib.h>

#define	LEN	32

int next(FILE *, char *);

int main(int argc, char **argv)
    {
    char to[14], from[14], buf[LEN + 1];
	char addrval[10];
    int addr = 0, chk, i, len;
    FILE *fp1, *fp2;

    if (argc <= 1) {
		printf("bin2hex: No file specified\n");
		printf("bin2hex <filename>.bin [starting address]\n");
		printf("filename of binary file, optional hexadecimal starting address (default 0)\n");
		printf("output is filename.hex of Intel hex records.\n");
		exit(1);
    }
	strcpy(from,argv[1]);
    strcat(from,".bin");
    if ((fp1 = fopen(from,"rb")) == NULL) {
        printf("bin2hex: Source file %s not found\n", from);
  	exit(1);
    }
    strcpy(to,argv[1]);
    strcat(to,".hex");
    if ((fp2 = fopen(to,"w")) == NULL) {
        printf("bin2hex: Could not create destination file %s\n", to);
  		exit(1);
    }
	if (argc > 2) { // address specification HRJ
		strcpy(addrval, argv[2]);
		// printf("bintohex: %s\n",addrval);
		if (i = sscanf(addrval,"%x",&addr) == EOF) {
	       printf("bin2hex: bad hex address on input\n");
  			exit(1);
		}
	}  // else default address addr is 0000H HRJ
	printf ("bin2hex: starting address is %X\n",addr);

    while ((len = next(fp1,buf)) != EOF)
	{
	if (len)
	    {
	    fprintf(fp2,":%02X%04X00",len,addr);
//	    printf(":%02X%04X00",len,addr);
	    chk = 0;
	    chk -= len;
	    chk -= addr & 0xff;
	    chk -= addr >> 8;
	    for (i=0; i<len; i++)
	        {
	        fprintf(fp2,"%02X",buf[i] & 0xff);
//	        printf("%02X",buf[i] & 0xff);
	        chk -= buf[i] & 0xff;
	        }
	    fprintf(fp2,"%02X\n",chk&0xff);
//    printf("%02X\n",chk&0xff); //test
	    }
	addr += LEN;
	if (addr > 0xFFFF) {
		addr = addr - 0x10000;
		printf("bin2hex: addr exceeds 0xFFFF, will wrap past 0x0000\n");
		}
	}
	printf ("bin2hex: ending address is %X\n",addr-1);
    fprintf(fp2,":00000001FF\n");
//    printf(":00000001FF\n");
    fclose(fp1);
    fclose(fp2);
    return 0;
    }

int next(FILE *fp1,char *buf)
    {
    int i, ch, flag = 1;
    for (i=0; i<LEN; i++)
	{
	ch = fgetc(fp1);
	if (ch == EOF)
	    return EOF;
	if (ch > 0)
     	  flag = 0;
	buf[i] = ch;
	}
    if (flag)
	return 0;
    return LEN;
    }

